/*
** Graphical User Interface using MUI
** for Multiface Two ACEpansion
*/

#include <clib/alib_protos.h>

#include <proto/exec.h>
#include <proto/utility.h>
#include <proto/intuition.h>
#include <proto/muimaster.h>

#include <acepansion/lib_header.h>

#include "acepansion.h"
#include "interface.h"
#define CATCOMP_NUMBERS
#include "generated/locale_strings.h" /* catalog string ids */



struct Library *IntuitionBase;
struct Library *UtilityBase;
struct Library *MUIMasterBase;



struct MUI_CustomClass *MCC_MultifaceTwoClass = NULL;



/*
** MCC_MultifaceTwoClass definitions
*/
#define MultifaceTwoObject NewObject(MCC_MultifaceTwoClass->mcc_Class, NULL

#define MUIA_MultifaceTwo_Plugin (TAG_USER | 0x0000) // [I..] struct ACEpansionPlugin *, Reference to the plugin managing the GUI (mandatory)

#define MUIM_MultifaceTwo_Notify (TAG_USER | 0x0100) // struct MUIP_MultifaceTwo_Notify, notify the plugin or the GUI about changes
    #define MUIV_MultifaceTwo_Notify_ToGUI    0 // Notify the GUI
    #define MUIV_MultifaceTwo_Notify_ToPlugin 1 // Notify the plugin
        // For MUIV_MultifaceTwo_Notify_ToPlugin only
        #define MUIV_MultifaceTwo_Notify_Reset 10 // Notify about RESET button pressed (no argument)
        #define MUIV_MultifaceTwo_Notify_Stop  11 // Notify about STOP button pressed (no argument)

struct MUIP_MultifaceTwo_Notify
{
    IPTR id;
    IPTR direction;
    IPTR type;
};


struct MultifaceTwoData
{
    struct ACEpansionPlugin *plugin;
};



/*
** Helper functions
*/
static ULONG red[3]    = { 0xffffffff,0x00000000,0x00000000 };
static ULONG yellow[3] = { 0xffffffff,0xffffffff,0x00000000 };

static Object * makeMF2Button(ULONG c[3], CONST_STRPTR t)
{
    return VGroup,
        ButtonFrame,
        MUIA_Font, MUIV_Font_Button,
        MUIA_Font         , MUIV_Font_Tiny,
        MUIA_InputMode    , MUIV_InputMode_RelVerify,
        MUIA_Background   , MUII_ButtonBack,
        MUIA_CycleChain   , TRUE,
        Child, HVSpace,
        Child, HGroup,
            Child, HVSpace,
            Child, ColorfieldObject,
                MUIA_FixWidthTxt, "OO",
                MUIA_FixHeightTxt, " ",
                MUIA_Colorfield_RGB, c,
                End,
            Child, HVSpace,
            End,
        Child, TextObject,
            MUIA_Text_PreParse, MUIX_C,
            MUIA_Text_Contents, t,
            End,
        Child, HVSpace,
        End;
}


/*
** MCC_MultifaceTwoClass methods
*/
static IPTR mNotify(struct IClass *cl, Object *obj, struct MUIP_MultifaceTwo_Notify *msg)
{
    struct MultifaceTwoData *data = INST_DATA(cl,obj);

    switch(msg->direction)
    {
        case MUIV_MultifaceTwo_Notify_ToGUI:
            break;
        case MUIV_MultifaceTwo_Notify_ToPlugin:
            switch(msg->type)
            {
                case MUIV_MultifaceTwo_Notify_Reset:
                    Plugin_TriggerReset(data->plugin);
                    break;
                case MUIV_MultifaceTwo_Notify_Stop:
                    Plugin_TriggerStop(data->plugin);
                    break;
            }
        break;
    }

    return 0;
}

static IPTR mNew(struct IClass *cl, Object *obj, Msg msg)
{

    struct TagItem *tags,*tag;
    struct MultifaceTwoData *data;

    struct ACEpansionPlugin *plugin = NULL;

    Object *BTN_Stop;
    Object *BTN_Reset;

    // Parse initial taglist
    for(tags=((struct opSet *)msg)->ops_AttrList; (tag=NextTagItem(&tags)); )
    {
        switch(tag->ti_Tag)
        {
            case MUIA_MultifaceTwo_Plugin:
                plugin = (struct ACEpansionPlugin *)tag->ti_Data;
                break;                              
        }
    }

    if(!plugin) return 0;

    obj = (Object *)DoSuperNew(cl,obj,
        GroupFrameT(GetString(MSG_CONTROLS)),
        MUIA_Group_Horiz, TRUE,
        Child, BTN_Stop  = makeMF2Button(red   , "  STOP  " ),
        Child, BTN_Reset = makeMF2Button(yellow, "  RESET  "),
        TAG_MORE,((struct opSet *)msg)->ops_AttrList);

    if(!obj) return 0;

    data = INST_DATA(cl,obj);

    data->plugin = plugin;

    // Buttons from GUI to plugin
    DoMethod(BTN_Reset, MUIM_Notify, MUIA_Pressed, FALSE,
             obj, 3, MUIM_MultifaceTwo_Notify, MUIV_MultifaceTwo_Notify_ToPlugin,
                     MUIV_MultifaceTwo_Notify_Reset);
    DoMethod(BTN_Stop, MUIM_Notify, MUIA_Pressed, FALSE,
             obj, 3, MUIM_MultifaceTwo_Notify, MUIV_MultifaceTwo_Notify_ToPlugin,
                     MUIV_MultifaceTwo_Notify_Stop);

    return (IPTR)obj;
}



/*
** MUI Custom Class dispatcher, creation & destruction
*/
DISPATCHER(MultifaceTwoClass)
{
    switch (msg->MethodID)
    {
        case OM_NEW                   : return(mNew   (cl,obj,(APTR)msg));
        case MUIM_MultifaceTwo_Notify : return(mNotify(cl,obj,(APTR)msg));
    }
}
DISPATCHER_END



/*
** Function which is called to initialize commons just
** after the library was loaded into memory and prior
** to any other API call
**
** This is the good place to open our required libraries
** and create our MUI custom classes
*/
BOOL GUI_InitResources(VOID)
{
    UtilityBase = OpenLibrary("utility.library", 0L);
    IntuitionBase = OpenLibrary("intuition.library", 0L);
    MUIMasterBase = OpenLibrary("muimaster.library", 0L);

    if(UtilityBase && IntuitionBase && MUIMasterBase)
    {
        MCC_MultifaceTwoClass = MUI_CreateCustomClass(
            NULL,
            MUIC_Group,
            NULL,
            sizeof(struct MultifaceTwoData),
            DISPATCHER_REF(MultifaceTwoClass));
    }

    return MCC_MultifaceTwoClass != NULL;
}

/*
** Function which is called to free commons just before
** the library is expurged from memory
**
** This is the good place to close our required libraries
** and destroy our MUI custom classes
*/
VOID GUI_FreeResources(VOID)
{
    if(MCC_MultifaceTwoClass)
        MUI_DeleteCustomClass(MCC_MultifaceTwoClass);

    CloseLibrary(MUIMasterBase);
    CloseLibrary(IntuitionBase);
    CloseLibrary(UtilityBase);
}



/*
** Function which is called from CreatePlugin() to build the GUI
*/
Object * GUI_Create(struct ACEpansionPlugin *plugin)
{
    return MultifaceTwoObject,
        MUIA_MultifaceTwo_Plugin, plugin,
        End;
}

/*
** Function which is called from DeletePlugin() to delete the GUI
*/
VOID GUI_Delete(Object *gui)
{
    MUI_DisposeObject(gui);
}



/*
** Functions called from the plugin
*/
VOID GUI_NotifyROMNotFound(UNUSED Object *gui, CONST_STRPTR rom)
{
    MUI_Request(NULL, NULL, 0, GetString(MSG_TITLE), GetString(MSG_OK), GetString(MSG_NO_ROM), rom);
}

